/*
 * Copyright 2011-13 Fraunhofer ISE
 *
 * This file is part of OpenMUC.
 * For more information visit http://www.openmuc.org
 *
 * OpenMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * OpenMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with OpenMUC.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.openmuc.framework.driver.spi;

/**
 * Class containing the following connection parameters.
 * <ul>
 * <li><b>interfaceAddress</b> The address of the interface</li>
 * <li><b>deviceAddress</b> The address of the device (e.g. on the local bus / ip network)</li>
 * <li><b>settings</b> A settings String which may contain several connection parameters (e.g.: baud, parity, stopbits
 * for serial connections)</li>
 * <li><b>connectionHandle</b>The connection handle</li>
 * </ul>
 */
public final class DeviceConnection {

	private final String interfaceAddress;
	private final String deviceAddress;
	private final String settings;
	private final Object connectionHandle;

	public DeviceConnection(String interfaceAddress, String deviceAddress, String settings, Object connectionHandle) {
		this.interfaceAddress = interfaceAddress;
		this.deviceAddress = deviceAddress;
		this.settings = settings;
		this.connectionHandle = connectionHandle;
	}

	public String getInterfaceAddress() {
		return interfaceAddress;
	}

	public String getDeviceAddress() {
		return deviceAddress;
	}

	public String getSettings() {
		return settings;
	}

	public Object getConnectionHandle() {
		return connectionHandle;
	}

}
