/*
 * Copyright 2011-13 Fraunhofer ISE
 *
 * This file is part of OpenMUC.
 * For more information visit http://www.openmuc.org
 *
 * OpenMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * OpenMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with OpenMUC.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openmuc.framework.driver.knx;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import org.openmuc.framework.data.Flag;
import org.openmuc.framework.data.Record;
import org.openmuc.framework.driver.spi.ChannelRecordContainer;
import org.openmuc.framework.driver.spi.RecordsReceivedListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import tuwien.auto.calimero.DetachEvent;
import tuwien.auto.calimero.GroupAddress;
import tuwien.auto.calimero.process.ProcessEvent;
import tuwien.auto.calimero.process.ProcessListener;

/**
 * @author frobra
 * 
 */
public class KnxProcessListener implements ProcessListener {

	private static Logger logger = LoggerFactory.getLogger(KnxProcessListener.class);

	private List<ChannelRecordContainer> containers;
	private RecordsReceivedListener listener;
	private final Hashtable<GroupAddress, byte[]> cachedValues;

	public KnxProcessListener() {
		cachedValues = new Hashtable<GroupAddress, byte[]>();

		containers = null;
		listener = null;
	}

	public void registerOpenMucListener(List<ChannelRecordContainer> containers, RecordsReceivedListener listener) {

		// for(ChannelRecordContainer container : containers) {
		// KnxGroupDP groupDP = (KnxGroupDP) container.getChannelHandle();
		// if(cachedValues.contains(groupDP)) {
		// groupDP.getKnxValue().setData(cachedValues.get(groupDP));
		// Record record = new Record(groupDP.getKnxValue().getOpenMucValue(), System.currentTimeMillis(),
		// Flag.NO_VALUE_RECEIVED_YET);;
		//
		// listener.newRecords(createNewRecords(container, record));
		// }
		// }

		this.containers = containers;
		this.listener = listener;
	}

	public synchronized void unregisterOpenMucListener() {
		containers = null;
		listener = null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see tuwien.auto.calimero.process.ProcessListener#groupWrite(tuwien.auto.calimero.process.ProcessEvent)
	 */
	@Override
	public void groupWrite(ProcessEvent e) {
		if (listener != null) {
			for (ChannelRecordContainer container : containers) {
				KnxGroupDP groupDP = (KnxGroupDP) container.getChannelHandle();
				if (groupDP.getMainAddress().equals(e.getDestination())) {
					groupDP.getKnxValue().setData(e.getASDU());
					logger.debug("Group write: " + e.getDestination());

					Record record = new Record(groupDP.getKnxValue().getOpenMucValue(), System.currentTimeMillis(),
							Flag.VALID);

					listener.newRecords(createNewRecords(container, record));

					break;
				}
			}
		}
		cachedValues.put(e.getDestination(), e.getASDU());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see tuwien.auto.calimero.process.ProcessListener#detached(tuwien.auto.calimero.DetachEvent)
	 */
	@Override
	public void detached(DetachEvent e) {

	}

	public Hashtable<GroupAddress, byte[]> getCachedValues() {
		return cachedValues;
	}

	public void invalidateCachedValue(GroupAddress groupAddress) {
		// cachedValues.remove(groupAddress);
		// if(listener != null) {
		// for(ChannelRecordContainer container : containers) {
		// KnxGroupDP groupDP = (KnxGroupDP) container.getChannelHandle();
		// if(groupDP.getMainAddress().equals(groupAddress)) {
		// logger.debug("Invalidating value for " + groupAddress);
		// Record record = new Record(null, System.currentTimeMillis(), Flag.NO_VALUE_RECEIVED_YET);
		//
		// listener.newRecords(createNewRecords(container, record));
		// }
		// }
		// }
	}

	private static List<ChannelRecordContainer> createNewRecords(ChannelRecordContainer container, Record record) {
		List<ChannelRecordContainer> recordContainers = new ArrayList<ChannelRecordContainer>();
		container.setRecord(record);
		recordContainers.add(container);
		return recordContainers;
	}

}
